import { Page, ElementHandle } from "puppeteer";
import * as assert from 'assert';
import { runPromises } from './utils';
import { getModuleOptions, getOptions } from './config';

const options = getOptions();
const moduleOptions = getModuleOptions();

/**
 * 点击 element
 * @param el 
 */
async function clickElement(el: ElementHandle | null) {
  if (!el) return;
  await el.click({ clickCount: options.uiConfig.clickCount })
}

/**
 * 输入框输入文本
 * @param inputEl 
 * @param text 
 */
async function typeInput(inputEl: ElementHandle, text: string) {
  await inputEl.type(text, { delay: options.uiConfig.typeDelay });
}

/**
 * 
 * @param selectorStr 
 * @param page 
 * @param text 
 */
async function typeElInput(selectorStr: string, page: Page, text: string) {
  let inputEl = await page.$(selectorStr);
  if (inputEl) {
    await typeInput(inputEl, text);
  }
}

/**
 * 点击 elClick
 * @param selectorStr 
 * @param page 
 */
async function clickElButton(selectorStr: string, page: Page) {
  let rootEl = await page.$(selectorStr);
  if (!rootEl) {
    return;
  }
  await clickElement(rootEl);
}

/**
 * 
 * @param page 
 * @param testId 
 * @param text 
 */
async function typeElSelect(selectorStr: string, page: Page, text: string) {
  let selectEl = await page.$(selectorStr);
  if (!selectEl) return;
  let inputEl = await selectEl.$(moduleOptions.elSelect.inputSelector);
  if (!inputEl) return;
  await typeInput(inputEl, text);
}

interface IVisibleInfo {
  visible: boolean
}
/**
 * 
 * @param nodes 
 */
function generateVisibleInfos(nodes: Array<Element>): Array<IVisibleInfo> {
  let resultArr: Array<IVisibleInfo> = [];
  nodes.forEach(node => {
    let nodeRect = node.getBoundingClientRect();
    let item = {
      visible: false
    };
    if (nodeRect && (nodeRect.width || nodeRect.height)) {
      item.visible = true;
    }
    resultArr.push(item);
  });
  return resultArr;
}

/**
 * 获取第一个可见的元素索引
 * @param {*} iSelectorStr 
 * @param {*} page 
 */
async function findVisibleElIndex(iSelectorStr: string, page: Page) {
  let selectorStr = iSelectorStr;
  let listVisibleArr = await page.$$eval(selectorStr, generateVisibleInfos);
  let visibleInx = listVisibleArr.findIndex((item) => item.visible);
  return visibleInx;
}

/**
 * 获取第一个 visible 的元素
 * @param iSelectorStr 
 * @param page 
 */
async function findVisibleEl(iSelectorStr: string, page: Page) {
  let visibleInx = await findVisibleElIndex(iSelectorStr, page);
  let items = await page.$$(iSelectorStr);
  let visibleItem = items[visibleInx];
  return visibleItem;
}

/**
 * 
 * @param page 
 */
async function findElDropDownItemEls(page: Page) {
  let dropDownSelectorStr = moduleOptions.elSelect.dropdownListSelector;
  let visibleInx = await findVisibleElIndex(dropDownSelectorStr, page);
  let listEls = await page.$$(dropDownSelectorStr);
  let currItem = listEls[visibleInx];
  let items = currItem.$$(moduleOptions.elSelect.dorpdownItemSelector);
  return items;
}

/**
 * 
 * @param selectorStr 
 * @param page 
 * @param inx 
 * @param emptySelectorStr 
 */
async function selectElSelectItem(selectorStr: string, page: Page, inx: number, emptySelectorStr: string) {
  let rootEl = await page.$(selectorStr);
  if (!rootEl) return;
  let inputEl = await rootEl.$(moduleOptions.elSelect.inputSelector)
  if (!inputEl) return;
  await inputEl.focus();
  await page.waitFor(options.uiConfig.uiWaitTime);
  let items = await findElDropDownItemEls(page);
  if (items && items.length) {
    let targetItem = items[inx];
    if (targetItem) {
      await targetItem.click();
      if (emptySelectorStr) {
        let otherEl = await page.$(emptySelectorStr);
        await clickElement(otherEl);
        await page.waitFor(options.uiConfig.uiWaitTime);
      }
    }
  }
}

/**
 * 
 * @param dropDownListEl 
 * @param colNum 
 * @param inx 
 */
async function selectElCascaderItem(dropDownListEl: ElementHandle, colNum: number, inx: number) {
  let menus = await dropDownListEl.$$(moduleOptions.elCascader.dropdownColSelector);
  let colMenu = menus[colNum];
  let colMenuItems = await colMenu.$$(moduleOptions.elCascader.dropdownColItemSelector);
  let targetItem = colMenuItems[inx];
  await clickElement(targetItem);
}

/**
 * 
 * @param selectotStr 
 * @param page 
 * @param vals 
 */
async function selectElCascaderItems(selectotStr: string, page: Page, vals: Array<number>) {
  let rootEl = await page.$(selectotStr);
  if (!rootEl) return;
  let inputEl = await rootEl.$(moduleOptions.elCascader.inputSelector);
  await clickElement(inputEl);
  await page.waitFor(options.uiConfig.uiWaitTime);
  let listEl = await findVisibleEl(moduleOptions.elCascader.dropdownListSelector, page)
  let promiseFns = vals.map((val, inx) => {
    return async () => {
      await selectElCascaderItem(listEl, inx, val);
      await page.waitFor(options.uiConfig.uiWaitTime);
    };
  })

  await runPromises(promiseFns);
}

/**
 * 
 * @param selectorStr 
 * @param page 
 * @param filePath 
 */
async function elUploadFiles(selectorStr: string, page: Page, filePath: Array<string>) {
  let rootEl = await page.$(selectorStr);
  if (!rootEl) return;
  let inputEl = await rootEl.$(moduleOptions.elUpload.inputSelector);
  if (inputEl) {
    await inputEl.uploadFile(...filePath);
  }
}

/**
 * 
 * @param selectorStr 
 * @param page 
 * @param inxs 
 */
async function checkElCheckboxGroup(selectorStr: string, page: Page, inxs: Array<number>) {
  let rootEl = await page.$(selectorStr);
  if (!rootEl) return;
  let inputEls = await rootEl.$$(moduleOptions.elCheckboxGroup.labelSelector);
  let promiseFns = inputEls.map((inputItem, inx) => {
    return async () => {
      if (inxs.includes(inx)) {
        await clickElement(inputItem);
      }
    }
  })
  await runPromises(promiseFns);
}


export default {
  clickElement,
  typeInput,
  runPromises,
  clickElButton,
  typeElInput,
  typeElSelect,
  selectElCascaderItems,
  checkElCheckboxGroup,
  elUploadFiles,
  selectElSelectItem
}




